@@ -4,13 +4,15 @@ use distribution_types::{CompatibleDist, IncompatibleDist, IncompatibleSource};
 use distribution_types::{DistributionMetadata, IncompatibleWheel, Name, PrioritizedDist};
 use pep440_rs::Version;
 use pep508_rs::MarkerEnvironment;
+use tracing::debug;
 use uv_normalize::PackageName;
+use uv_types::InstalledPackagesProvider;
 
 use crate::preferences::Preferences;
 use crate::prerelease_mode::PreReleaseStrategy;
 use crate::resolution_mode::ResolutionStrategy;
 use crate::version_map::{VersionMap, VersionMapDistHandle};
-use crate::{Manifest, Options};
+use crate::{Exclusions, Manifest, Options};
 
 #[derive(Debug, Clone)]
 pub(crate) struct CandidateSelector {
@@ -61,23 +63,63 @@ enum AllowPreRelease {
 
 impl CandidateSelector {
     /// Select a [`Candidate`] from a set of candidate versions and files.
-    pub(crate) fn select<'a>(
+    ///
+    /// Unless present in the provided [`Exclusions`], local distributions from the
+    /// [`InstalledPackagesProvider`] are preferred over remote distributions in
+    /// the [`VersionMap`].
+    pub(crate) fn select<'a, InstalledPackages: InstalledPackagesProvider>(
         &'a self,
         package_name: &'a PackageName,
         range: &'a Range<Version>,
         version_map: &'a VersionMap,
         preferences: &'a Preferences,
+        installed_packages: &'a InstalledPackages,
+        exclusions: &'a Exclusions,
     ) -> Option<Candidate<'a>> {
         // If the package has a preference (e.g., an existing version from an existing lockfile),
         // and the preference satisfies the current range, use that.
         if let Some(version) = preferences.version(package_name) {
             if range.contains(version) {
+                // Check for a locally installed distribution that matches the preferred version
+                if !exclusions.contains(package_name) {
+                    for dist in installed_packages.get_packages(package_name) {
+                        if dist.version() == version {
+                            debug!("Found installed version of {dist} that satisfies preference in {range}");
+
+                            return Some(Candidate {
+                                name: package_name,
+                                version,
+                                dist: CandidateDist::Compatible(CompatibleDist::InstalledDist(
+                                    dist,
+                                )),
+                            });
+                        }
+                    }
+                }
+
+                // Check for a remote distribution that matches the preferred version
                 if let Some(file) = version_map.get(version) {
                     return Some(Candidate::new(package_name, version, file));
                 }
             }
         }
 
+        // Check for a locally installed distribution that satisfies the range
+        if !exclusions.contains(package_name) {
+            for dist in installed_packages.get_packages(package_name) {
+                let version = dist.version();
+                if range.contains(version) {
+                    debug!("Found installed version of {dist} that satisfies {range}");
+
+                    return Some(Candidate {
+                        name: package_name,
+                        version,
+                        dist: CandidateDist::Compatible(CompatibleDist::InstalledDist(dist)),
+                    });
+                }
+            }
+        }
+
         // Determine the appropriate prerelease strategy for the current package.
         let allow_prerelease = match &self.prerelease_strategy {
             PreReleaseStrategy::Disallow => AllowPreRelease::No,
@@ -100,7 +142,7 @@ impl CandidateSelector {
         };
 
         tracing::trace!(
-            "selecting candidate for package {:?} with range {:?} with {} versions",
+            "selecting candidate for package {:?} with range {:?} with {} remote versions",
             package_name,
             range,
             version_map.len()
