@@ -8,7 +8,9 @@ use pubgrub::range::Range;
 use pubgrub::report::{DefaultStringReporter, DerivationTree, Reporter};
 use rustc_hash::FxHashMap;
 
-use distribution_types::{BuiltDist, IndexLocations, PathBuiltDist, PathSourceDist, SourceDist};
+use distribution_types::{
+    BuiltDist, IndexLocations, InstalledDist, PathBuiltDist, PathSourceDist, SourceDist,
+};
 use once_map::OnceMap;
 use pep440_rs::Version;
 use pep508_rs::Requirement;
@@ -70,6 +72,10 @@ pub enum ResolveError {
     #[error("Failed to read: {0}")]
     Read(Box<PathBuiltDist>, #[source] uv_distribution::Error),
 
+    // TODO(zanieb): Use `thiserror` in `InstalledDist` so we can avoid chaining `anyhow`
+    #[error("Failed to read metadata from installed package: {0}")]
+    ReadInstalled(Box<InstalledDist>, #[source] anyhow::Error),
+
     #[error("Failed to build: {0}")]
     Build(Box<PathSourceDist>, #[source] uv_distribution::Error),
 
