@@ -2,9 +2,9 @@ use distribution_types::LocalEditable;
 use pep508_rs::{MarkerEnvironment, Requirement};
 use pypi_types::Metadata23;
 use uv_normalize::PackageName;
-use uv_types::RequestedRequirements;
+use uv_types::{Constraints, Overrides, RequestedRequirements};
 
-use crate::preferences::Preference;
+use crate::{preferences::Preference, Exclusions};
 
 /// A manifest of requirements, constraints, and preferences.
 #[derive(Clone, Debug)]
@@ -13,10 +13,10 @@ pub struct Manifest {
     pub(crate) requirements: Vec<Requirement>,
 
     /// The constraints for the project.
-    pub(crate) constraints: Vec<Requirement>,
+    pub(crate) constraints: Constraints,
 
     /// The overrides for the project.
-    pub(crate) overrides: Vec<Requirement>,
+    pub(crate) overrides: Overrides,
 
     /// The preferences for the project.
     ///
@@ -34,6 +34,12 @@ pub struct Manifest {
     /// direct requirements in their own right.
     pub(crate) editables: Vec<(LocalEditable, Metadata23)>,
 
+    /// The installed packages to exclude from consideration during resolution.
+    ///
+    /// These typically represent packages that are being upgraded or reinstalled
+    /// and should be pulled from a remote source like a package index.
+    pub(crate) exclusions: Exclusions,
+
     /// The lookahead requirements for the project.
     ///
     /// These represent transitive dependencies that should be incorporated when making
@@ -43,13 +49,15 @@ pub struct Manifest {
 }
 
 impl Manifest {
+    #[allow(clippy::too_many_arguments)]
     pub fn new(
         requirements: Vec<Requirement>,
-        constraints: Vec<Requirement>,
-        overrides: Vec<Requirement>,
+        constraints: Constraints,
+        overrides: Overrides,
         preferences: Vec<Preference>,
         project: Option<PackageName>,
         editables: Vec<(LocalEditable, Metadata23)>,
+        exclusions: Exclusions,
         lookaheads: Vec<RequestedRequirements>,
     ) -> Self {
         Self {
@@ -59,6 +67,7 @@ impl Manifest {
             preferences,
             project,
             editables,
+            exclusions,
             lookaheads,
         }
     }
@@ -66,11 +75,12 @@ impl Manifest {
     pub fn simple(requirements: Vec<Requirement>) -> Self {
         Self {
             requirements,
-            constraints: Vec::new(),
-            overrides: Vec::new(),
+            constraints: Constraints::default(),
+            overrides: Overrides::default(),
             preferences: Vec::new(),
             project: None,
             editables: Vec::new(),
+            exclusions: Exclusions::default(),
             lookaheads: Vec::new(),
         }
     }
@@ -90,30 +100,28 @@ impl Manifest {
         self.lookaheads
             .iter()
             .flat_map(|lookahead| {
-                lookahead
-                    .requirements()
-                    .iter()
+                self.overrides
+                    .apply(lookahead.requirements())
                     .filter(|requirement| requirement.evaluate_markers(markers, lookahead.extras()))
             })
             .chain(self.editables.iter().flat_map(|(editable, metadata)| {
-                metadata
-                    .requires_dist
-                    .iter()
+                self.overrides
+                    .apply(&metadata.requires_dist)
                     .filter(|requirement| requirement.evaluate_markers(markers, &editable.extras))
             }))
             .chain(
-                self.requirements
-                    .iter()
+                self.overrides
+                    .apply(&self.requirements)
                     .filter(|requirement| requirement.evaluate_markers(markers, &[])),
             )
             .chain(
                 self.constraints
-                    .iter()
+                    .requirements()
                     .filter(|requirement| requirement.evaluate_markers(markers, &[])),
             )
             .chain(
                 self.overrides
-                    .iter()
+                    .requirements()
                     .filter(|requirement| requirement.evaluate_markers(markers, &[])),
             )
     }
@@ -130,22 +138,31 @@ impl Manifest {
         self.lookaheads
             .iter()
             .flat_map(|lookahead| {
-                lookahead
-                    .requirements()
-                    .iter()
+                self.overrides
+                    .apply(lookahead.requirements())
                     .filter(|requirement| requirement.evaluate_markers(markers, lookahead.extras()))
             })
             .chain(self.editables.iter().flat_map(|(editable, metadata)| {
-                metadata
-                    .requires_dist
-                    .iter()
+                self.overrides
+                    .apply(&metadata.requires_dist)
                     .filter(|requirement| requirement.evaluate_markers(markers, &editable.extras))
             }))
             .chain(
-                self.requirements
-                    .iter()
+                self.overrides
+                    .apply(&self.requirements)
                     .filter(|requirement| requirement.evaluate_markers(markers, &[])),
             )
             .map(|requirement| &requirement.name)
     }
+
+    /// Apply the overrides and constraints to a set of requirements.
+    ///
+    /// Constraints are always applied _on top_ of overrides, such that constraints are applied
+    /// even if a requirement is overridden.
+    pub fn apply<'a>(
+        &'a self,
+        requirements: impl IntoIterator<Item = &'a Requirement>,
+    ) -> impl Iterator<Item = &Requirement> {
+        self.constraints.apply(self.overrides.apply(requirements))
+    }
 }
