@@ -8,6 +8,7 @@ use pep508_rs::{
 };
 use pypi_types::{HashError, Hashes};
 use requirements_txt::RequirementEntry;
+use tracing::trace;
 use uv_normalize::PackageName;
 
 #[derive(thiserror::Error, Debug)]
@@ -68,13 +69,17 @@ impl Preference {
 pub(crate) struct Preferences(FxHashMap<PackageName, Pin>);
 
 impl Preferences {
-    /// Create a map of pinned packages from a list of [`Preference`] entries.
-    pub(crate) fn from_requirements(
-        requirements: Vec<Preference>,
+    /// Create a map of pinned packages from an iterator of [`Preference`] entries.
+    /// Takes ownership of the [`Preference`] entries.
+    ///
+    /// The provided [`MarkerEnvironment`] will be used to filter  the preferences
+    /// to an applicable subset.
+    pub(crate) fn from_iter<PreferenceIterator: IntoIterator<Item = Preference>>(
+        preferences: PreferenceIterator,
         markers: &MarkerEnvironment,
     ) -> Self {
         Self(
-            requirements
+            preferences
                 .into_iter()
                 .filter_map(|preference| {
                     let Preference {
@@ -84,26 +89,45 @@ impl Preferences {
 
                     // Search for, e.g., `flask==1.2.3` entries that match the current environment.
                     if !requirement.evaluate_markers(markers, &[]) {
+                        trace!(
+                            "Excluding {requirement} from preferences due to unmatched markers."
+                        );
                         return None;
                     }
-                    let Some(VersionOrUrl::VersionSpecifier(version_specifiers)) =
-                        requirement.version_or_url.as_ref()
-                    else {
-                        return None;
-                    };
-                    let [version_specifier] = version_specifiers.as_ref() else {
-                        return None;
-                    };
-                    if *version_specifier.operator() != Operator::Equal {
-                        return None;
+                    match requirement.version_or_url.as_ref() {
+                        Some(VersionOrUrl::VersionSpecifier(version_specifiers)) =>
+                         {
+                            let [version_specifier] = version_specifiers.as_ref() else {
+                                trace!(
+                                    "Excluding {requirement} from preferences due to multiple version specifiers."
+                                );
+                                return None;
+                            };
+                            if *version_specifier.operator() != Operator::Equal {
+                                trace!(
+                                    "Excluding {requirement} from preferences due to inexact version specifier."
+                                );
+                                return None;
+                            }
+                            Some((
+                                requirement.name,
+                                Pin {
+                                    version: version_specifier.version().clone(),
+                                    hashes,
+                                },
+                            ))
+                        }
+                        Some(VersionOrUrl::Url(_)) => {
+                            trace!(
+                                "Excluding {requirement} from preferences due to URL dependency."
+                            );
+                            None
+                        }
+                        _ => {
+                        None
                     }
-                    Some((
-                        requirement.name,
-                        Pin {
-                            version: version_specifier.version().clone(),
-                            hashes,
-                        },
-                    ))
+                    }
+
                 })
                 .collect(),
         )
