@@ -6,9 +6,8 @@ use distribution_types::Verbatim;
 use pep440_rs::Version;
 use pep508_rs::{MarkerEnvironment, Requirement, VersionOrUrl};
 use uv_normalize::{ExtraName, PackageName};
+use uv_types::{Constraints, Overrides};
 
-use crate::constraints::Constraints;
-use crate::overrides::Overrides;
 use crate::pubgrub::specifier::PubGrubSpecifier;
 use crate::pubgrub::PubGrubPackage;
 use crate::resolver::{Locals, Urls};
