@@ -24,7 +24,7 @@ use uv_client::{
 };
 use uv_dispatch::BuildDispatch;
 use uv_fs::Simplified;
-use uv_installer::{Downloader, NoBinary};
+use uv_installer::Downloader;
 use uv_interpreter::{find_best_python, PythonEnvironment, PythonVersion};
 use uv_normalize::{ExtraName, PackageName};
 use uv_requirements::{
@@ -32,10 +32,13 @@ use uv_requirements::{
     RequirementsSource, RequirementsSpecification, SourceTreeResolver,
 };
 use uv_resolver::{
-    AnnotationStyle, DependencyMode, DisplayResolutionGraph, InMemoryIndex, Manifest,
+    AnnotationStyle, DependencyMode, DisplayResolutionGraph, Exclusions, InMemoryIndex, Manifest,
     OptionsBuilder, PreReleaseMode, PythonRequirement, ResolutionMode, Resolver,
 };
-use uv_types::{BuildIsolation, ConfigSettings, InFlight, NoBuild, SetupPyStrategy, Upgrade};
+use uv_types::{
+    BuildIsolation, ConfigSettings, Constraints, EmptyInstalledPackages, InFlight, NoBinary,
+    NoBuild, Overrides, SetupPyStrategy, Upgrade,
+};
 use uv_warnings::warn_user;
 
 use crate::commands::reporters::{DownloadReporter, ResolverReporter};
@@ -215,6 +218,10 @@ pub(crate) async fn pip_compile(
     // Read the lockfile, if present.
     let preferences = read_lockfile(output_file, upgrade).await?;
 
+    // Collect constraints and overrides.
+    let constraints = Constraints::from_requirements(constraints);
+    let overrides = Overrides::from_requirements(overrides);
+
     // Resolve the flat indexes from `--find-links`.
     let flat_index = {
         let client = FlatIndexClient::new(&client, &cache);
@@ -274,14 +281,8 @@ pub(crate) async fn pip_compile(
         requirements
     };
 
-    // Determine any lookahead requirements.
-    let lookaheads = LookaheadResolver::new(&requirements)
-        .with_reporter(ResolverReporter::from(printer))
-        .resolve(&build_dispatch, &client)
-        .await?;
-
     // Build the editables and add their requirements
-    let editable_metadata = if editables.is_empty() {
+    let editables = if editables.is_empty() {
         Vec::new()
     } else {
         let start = std::time::Instant::now();
@@ -336,6 +337,12 @@ pub(crate) async fn pip_compile(
         editables
     };
 
+    // Determine any lookahead requirements.
+    let lookaheads = LookaheadResolver::new(&requirements, &constraints, &overrides, &editables)
+        .with_reporter(ResolverReporter::from(printer))
+        .resolve(&build_dispatch, &markers, &client)
+        .await?;
+
     // Create a manifest of the requirements.
     let manifest = Manifest::new(
         requirements,
@@ -343,7 +350,9 @@ pub(crate) async fn pip_compile(
         overrides,
         preferences,
         project,
-        editable_metadata,
+        editables,
+        // Do not consider any installed packages during compilation
+        Exclusions::All,
         lookaheads,
     );
 
@@ -365,6 +374,7 @@ pub(crate) async fn pip_compile(
         &flat_index,
         &top_level_index,
         &build_dispatch,
+        &EmptyInstalledPackages,
     )?
     .with_reporter(ResolverReporter::from(printer));
 
